package cc.smtweb.system.bpm.web.engine.flow;

import cc.smtweb.framework.core.exception.BizException;
import cc.smtweb.system.bpm.web.design.flow.FlowConst;
import cc.smtweb.system.bpm.web.design.flow.ModelProcHelper;
import cc.smtweb.system.bpm.web.design.flow.define.Activity;
import cc.smtweb.system.bpm.web.design.flow.define.ProcInfo;
import cc.smtweb.system.bpm.web.design.flow.define.Trans;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Akmm at 13-6-24 下午9:42
 * 单据
 */
public class FlowHelper {
    /**
     * 得到指定活动的下一任务节点
     *
     * @param act_id
     * @return
     */
    public static List<Activity> getNextActivity(FlowInstance flowInstance) throws Exception {
        Activity act = ModelProcHelper.getNextActivity(flowInstance.act_inst.getPrcId(), flowInstance.act_inst.getActId());
        if (act == null) {
            throw new BizException("未找到可用的活动节点定义！");
        }
        List<Activity> listRet = new ArrayList<>();
        buildNextActivityRes(act, flowInstance, listRet);
        if (listRet.isEmpty()) {
            throw new BizException("未找到可用的活动节点定义！");
        }
        return listRet;
    }

    /**
     * 递归处理，兼容判断后面加判断，并发后面跟并发
     *
     * @param act
     * @param flowInstance
     * @param listRet
     * @throws Exception
     */
    private static void buildNextActivityRes(Activity act, FlowInstance flowInstance, List<Activity> listRet) throws Exception {
        //待返回的结果集
//        if (act.getActType() == FlowConst.ActivityType.scriptTask) 自动任务暂不支持
        ProcInfo proc = flowInstance.proc_def.getProcInfo();
        if (act.getType() == FlowConst.ActivityType.CONDITION.value) {//条件分支
            List<Trans> list = ModelProcHelper.getNextTrans(flowInstance.proc_inst.getPrcId(), act.getId());
            for (Trans t : list) {
                if (flowInstance.execTrans(t.getExpr())) {
                    Activity a = proc.findActivity(t.getDst());
                    if (a == null) {
                        throw new BizException("无效的活动节点定义（" + t.getDst() + "）！");
                    }
                    buildNextActivityRes(a, flowInstance, listRet);
                    break;
                }
            }
        } else if (act.getType() == FlowConst.ActivityType.PARALLEL.value) {//并发分支
            List<Trans> list = ModelProcHelper.getNextTrans(flowInstance.proc_def.getId(), act.getId());
            for (Trans t : list) {
                if (flowInstance.execTrans(t.getExpr())) {
                    Activity a = proc.findActivity(t.getDst());
                    if (a == null) {
                        throw new BizException("无效的活动节点定义（" + t.getDst() + "）！");
                    }
                    buildNextActivityRes(a, flowInstance, listRet);
                }
            }
        } else {
            listRet.add(act);
        }
    }

}
